/*
 * check_msword - check that a specified file really is an MSWord file, and
 *		  that it contains no macros (and hence: no viruses).
 *
 * Copyright (c) Christopher M. R. Lowth - chris@lowth.com
 * Written by Christopher M. R. Lowth
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. The author is not responsible for the consequences of use of this
 *    software, no matter how awful, even if they arise from flaws in it.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Since few users ever read sources,
 *    credits must appear in the documentation.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.  Since few users
 *    ever read sources, credits must appear in the documentation.
 *
 * 4. This notice may not be removed or altered.
 */

/******************************************************************************

This program checks that the file named as it's argument is an MSword
document, and that is contains no macros. It does this by searching the
file for a set of strings, and confirming that these stings either exist or
dont in fixed combinations.

The result of the program's deliberations are output on stdout as commands
that can be executed by the "part_filter" shell script that is also part of
the protector suite.

The program does not actually understand the format of MSWord documents.
The logic for performing the checks described above has been derived by
inspecting a number of document files - and is probably rather too naive
for long term use. Consider this to be an over-simple prototype for future
improvement.

******************************************************************************/


#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#define TRUE 1
#define FALSE 0

/******************************************************************************/

char *check_document(char *fname)
{
	int id;
	char buff[16 * 1024];
	char c;

	char	k1[] = "R\0o\0o\0t\0 \0E\0n\0t\0r\0y",
		k2[] = "O\0b\0j\0e\0c\0t\0P\0o\0o\0l",
		k3[] = "W\0o\0r\0d\0D\0o\0c\0u\0m\0e\0n\0t",
		k4[] = "M\0a\0c\0r\0o\0s",
		k6[] = "MSWordDoc",
		k7[] = "Word.Document.6",
		k9[] = "Word.Document.8";

	int	k1_found = FALSE,
		k2_found = FALSE,
		k3_found = FALSE,
		k4_found = FALSE,
		k6_found = FALSE,
		k7_found = FALSE,
		k9_found = FALSE;

	int len, i;
	int version = 0;
	long key;

	if ((id = open(fname, O_RDONLY)) == -1)
		return "cant_read_decoded";

	if (read(id, &key, sizeof(key)) != sizeof(key)) {
		close(id);
		return "cant_read_decoded";
	}

	if (key == 0x002DA5DB)
		version = 2;

	if (key != 0x002DA5DB && key != 0xE011CFD0) {
		close(id);
		return "wrong_type";
	}

	for ( ; ; ) {
		memset(buff, 0, sizeof(buff));
		len = read(id, buff, sizeof(buff));
		for (i=0; i < (sizeof(buff)-24); i++) {
			c = buff[i];

			if (c == k1[0] && !memcmp(buff+i, k1, sizeof(k1)))
				k1_found = TRUE;

			if (c == k2[0] && !memcmp(buff+i, k2, sizeof(k2)))
				k2_found = TRUE;

			if (c == k3[0] && !memcmp(buff+i, k3, sizeof(k3)))
				k3_found = TRUE;

			if (c == k4[0] && !memcmp(buff+i, k4, sizeof(k4)))
				k4_found = TRUE;

			if (c == k6[0] && !memcmp(buff+i, k6, sizeof(k6)))
				k6_found = TRUE;

			if (c == k7[0] && !memcmp(buff+i, k7, sizeof(k7)))
				k7_found = TRUE;

			if (c == k9[0] && !memcmp(buff+i, k9, sizeof(k9)))
				k9_found = TRUE;

		}
		if (len != sizeof(buff))
			break;
		lseek(id, -32, SEEK_CUR);
	}
	close(id);

	if (version == 0 && k6_found && k7_found)
		version = 6;

	if (version == 0 && k6_found && k9_found)
		version = 8;

	if (version == 0 && k1_found && k2_found && k3_found)
		version = 1;	/* Dont know what this is, but I found it in the
				   field */

	if (version == 0)
		return "wrong_type";

	if (k4_found)
		return "msword_has_macros";

	return NULL;
}

/***************************************************************************************/

int main(int argc, char *argv[])
{
	char *err;

	assert(argc == 2);

	err = check_document(argv[1]);

	if (err == NULL) {
		printf("allow_unchanged\n");
		return 0;
	}
	else {
		printf("replace_with %s\n", err);
		return 2;
	}
}
