/*
 * Copyright (c) Christopher M. R. Lowth - chris@lowth.com
 * Written by Christopher M. R. Lowth
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. The author is not responsible for the consequences of use of this
 *    software, no matter how awful, even if they arise from flaws in it.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Since few users ever read sources,
 *    credits must appear in the documentation.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.  Since few users
 *    ever read sources, credits must appear in the documentation.
 *
 * 4. This notice may not be removed or altered.
 */

#define TRUE	1
#define FALSE	0

extern int local_errno;

/****************************************************************************
 * Email headers are held internally as a set of "fields" - a value text,
 * indexed by a textual "key"
 */

typedef struct field_s {
	char	*key;
	char	*value;
	struct field_s *next;
} field_t;

/****************************************************************************
 * An e-mail record consists of four "zones"..
 *
 *	zone 1:		A set of headers (terminated by an empty line).
 *
 *	zone 2:		Some "internal" text (not part of any "parts").
 *
 *	zone 3:		Zero or more "parts" - delimited by a boundary.
 *
 *	zone 4:		Some further "internal" text after the end of the
 *			last "part".
 */

typedef struct mailrec_s {
	FILE	*file;
	long	start_header;
	long	start_body;
	long	start_parts;
	long	end;
	field_t	*headers;
	char	*boundary;
	int	last_part;
	struct mailrec_s *parts;
	struct mailrec_s *next;
} mailrec_t;


/****************************************************************************
 * Mail record reading -- read.c
 */

/* mailrec_t *read_mailrec(FILE *source, char *boundary); */


#define E_NO_COLON		-1000
#define E_OUT_OF_MEMORY		-1001
#define E_BAD_HEADER		-1002
#define E_EMPTY_STRING		-1003
#define E_HEADER_NOT_FOUND	-1004
#define E_SUB_HEADER_NOT_FOUND	-1005
