/*
 * qpdecode - Decode quoted-printable encoded text.
 *
 * Copyright (c) Christopher M. R. Lowth - chris@lowth.com
 * Written by Christopher M. R. Lowth
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. The author is not responsible for the consequences of use of this
 *    software, no matter how awful, even if they arise from flaws in it.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Since few users ever read sources,
 *    credits must appear in the documentation.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.  Since few users
 *    ever read sources, credits must appear in the documentation.
 *
 * 4. This notice may not be removed or altered.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

static int hex_digit_value(int c)
{
	if (c >= '0' && c <= '9')
		return c - '0';

	if (c >= 'a' && c <= 'f')
		return c - 'a' + 10;

	if (c >= 'A' && c <= 'F')
		return c - 'A' + 10;

	return 0;
}

int main(int argc, char *argv[])
{
	int i, len;
	char buff[2048];
	char *eol;

	while (fgets(buff, sizeof(buff), stdin) != NULL) {
		/*
		 * Trailing white space must be ignored
		 */

		len = strlen(buff);
		while (len > 0 && strchr(" \n\r\t", buff[len-1]) != NULL)
			len --;
		buff[len] = '\0';

		/*
		 * Parse what is left of the line
		 */

		eol = "\n";
		for (i=0; i<len; i++) {
			/*
			 * An "=" followed by two hex digits, is converted
			 * back into the relevant ascii character.
			 */

			if (
				buff[i] == '=' &&
				isxdigit(buff[i+1]) &&
				isxdigit(buff[i+2])
			) {
				putchar(
					(hex_digit_value(buff[i+1]) * 16) +
					hex_digit_value(buff[i+2])
				);
				i += 2;
			}

			/*
			 * An "=" at the end of the line means that the
			 * next line is a continuation of this one -- so:
			 * no EOL sequence is output.
			 */

			else if (
				buff[i] == '=' &&
				i == len - 1
			)
				eol = "";

			/*
			 * Anything else is simply copied out "verbatim"
			 */

			else
				putchar(buff[i]);
		}
		printf(eol);
	}

	return 0;
}
