/*
 * Copyright (c) Christopher M. R. Lowth - chris@lowth.com
 * Written by Christopher M. R. Lowth
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. The author is not responsible for the consequences of use of this
 *    software, no matter how awful, even if they arise from flaws in it.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Since few users ever read sources,
 *    credits must appear in the documentation.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.  Since few users
 *    ever read sources, credits must appear in the documentation.
 *
 * 4. This notice may not be removed or altered.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "protector.h"
#include "proto.h"
#include "paths.h"

#define ARGV0 "revive"
#define MMENCODE "/usr/bin/mmencode"
#define UUDECODE "/usr/bin/uudecode"

int local_errno = 0;

int call_mmencode(FILE *src, char *tgtName, char *flag)
{
	char cmd[1024];
	char buff[1024];
	FILE *tgt;

	if (access(MMENCODE, X_OK) != 0) {
		fprintf(stderr, ARGV0 ": " MMENCODE " not executable\n");
		return 2;
	}
	sprintf(cmd, MMENCODE " -u %s -o '%s'", flag, tgtName);

	tgt = popen(cmd, "w");

	while (fgets(buff, sizeof(buff), src) != NULL) {
		fputs(buff, tgt);
	}
	pclose(tgt);

	return 0;
}

int decode_base64(FILE *src, char *tgtName)
{
	return call_mmencode(src, tgtName, "");
}

int decode_quoted_printable(FILE *src, char *tgtName)
{
	return call_mmencode(src, tgtName, "-q");
}

int decode_uuencoded(FILE *src, char *tgtName)
{
	char cmd[1024];
	char buff[1024];
	FILE *tgt;

	if (access(UUDECODE, X_OK) != 0) {
		fprintf(stderr, ARGV0 ": " UUDECODE " not executable\n");
		return 2;
	}

	sprintf(cmd, UUDECODE " -o '%s'", tgtName);

	tgt = popen(cmd, "w");

	while (fgets(buff, sizeof(buff), src) != NULL) {
		fputs(buff, tgt);
	}
	pclose(tgt);

	return 0;
}

int copy_as_is(FILE *src, char *tgtName)
{
	char buff[1024];
	FILE *tgt = fopen(tgtName, "w");

	if (tgt == NULL) {
		fprintf(stderr, ARGV0 ": Cant open %s\n", tgtName);
		return 2;
	}

	while (fgets(buff, sizeof(buff), src) != NULL) {
		fputs(buff, tgt);
	}
	fclose(tgt);

	return 0;
}


int main(int argc, char *argv[])
{
	mailrec_t *mailrec = malloc(sizeof(mailrec_t));
	char *srcName, *cte;
	FILE *srcId;

	if (argc != 3) {
		fprintf(stderr, ARGV0 ": Usage: " ARGV0 " source target\n");
		return 2;
	}

	if (mailrec == NULL) {
		fprintf(stderr, ARGV0 ": Out of memory\n");
		return 2;
	}

	if (strchr(argv[1], '/') == NULL) {
		srcName = malloc(strlen(argv[1]) + strlen(REJ_DIR) + 2);
		if (srcName == NULL) {
			fprintf(stderr, ARGV0 ": Out of memory\n");
			return 2;
		}
		strcpy(srcName, REJ_DIR "/");
		strcat(srcName, argv[1]);
	}
	else
		srcName = argv[1];

	srcId = fopen(srcName, "r");
	if (srcId == NULL) {
		fprintf(stderr, ARGV0 ": Cant open %s\n", srcName);
		return 2;
	}

	memset(mailrec, 0, sizeof(mailrec_t));
	mailrec->file = srcId;
	mailrec->start_header = 0;

	if (!read_headers(srcId, mailrec)) {
		fprintf(stderr, ARGV0 ": Cant read mime headers in %s\n", argv[1]);
		return 2;
	}

	mailrec->start_body = ftell(srcId);

	cte = get_header(mailrec, "content-transfer-encoding", 1);

	if (strcasecmp(cte, "base64") == 0)
		return decode_base64(srcId, argv[2]);
	else if (strcasecmp(cte, "7bit") == 0)
		return copy_as_is(srcId, argv[2]);
	else if (strcasecmp(cte, "8bit") == 0)
		return copy_as_is(srcId, argv[2]);
	else if (strcasecmp(cte, "binary") == 0)
		return copy_as_is(srcId, argv[2]);
	else if (strcasecmp(cte, "quoted-printable") == 0)
		return decode_quoted_printable(srcId, argv[2]);
	else if (strcasecmp(cte, "x-uuencode") == 0)
		return decode_uuencoded(srcId, argv[2]);
	else {
		fprintf(stderr, ARGV0 ": Unknown content-transfer-encoding: %s\n", cte);
		return 2;
	}
}
