/*
 * Copyright (c) Christopher M. R. Lowth - chris@lowth.com
 * Written by Christopher M. R. Lowth
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. The author is not responsible for the consequences of use of this
 *    software, no matter how awful, even if they arise from flaws in it.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Since few users ever read sources,
 *    credits must appear in the documentation.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.  Since few users
 *    ever read sources, credits must appear in the documentation.
 *
 * 4. This notice may not be removed or altered.
 */

#include <stdio.h>
#include <time.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>
#include <string.h>

#ifndef REJ_DIR
#error "REJ_DIR is not defined"
#endif

int get_file_name()
{
	time_t t = time(NULL);
	struct tm *tm = localtime(&t);
	char base_name[80];
	char file_name[80];
	int n;
	int id;

	sprintf(base_name, REJ_DIR "/%04d%02d%02d.%02d%02d%02d",
		tm->tm_year + 1900,
		tm->tm_mon + 1,
		tm->tm_mday,
		tm->tm_hour,
		tm->tm_min,
		tm->tm_sec
	);

	for (n=1; ; n++) {
		sprintf(file_name, "%s.%d", base_name, n);
		id = open(file_name, O_CREAT | O_EXCL | O_RDWR, 0666);
		if (id >= 0) {
			fchown(id, getuid(), getgid());
			close(id);
			printf("%s\n", file_name);
			return 0;
		}
		assert(errno == EEXIST);
	}

	return 2;
}

int write_file(char *file_name)
{
	char *dir = REJ_DIR;
	int id;
	int len;
	char buff[8192];

	assert(memcmp(file_name, dir, strlen(dir)) == 0);

	id = open(file_name, O_WRONLY | O_TRUNC);
	assert(id != -1);

	while ((len = read(0, buff, sizeof(buff))) > 0)
		write(id, buff, len);

	fchown(id, getuid(), getgid());
	close(id);

	return 0;
}

int main(int argc, char *argv[])
{
	if (argc == 2 && strcmp(argv[1], "-n") == 0)
		return get_file_name();

	if (argc == 3 && strcmp(argv[1], "-w") == 0)
		return write_file(argv[2]);

	fprintf(stderr, "Syntax error\n");
	return 2;
}
